<?php
require_once 'PHPOF2/DB.php';
require_once 'PHPOF2/DBTable.php';
require_once 'PHPOF2/DBRow.php';

if (!isset($argv[1])) {
	die("Usage: $argv[0] [dsn]\n");
}
// The DSN to the main application database
$dsn = $argv[1];

class PHPOF2_LOB_Test
{
	public $events = array();
	
	public function migrateLOBStorageCallback($serialised_prikey, $column, $event, $extra)
	{
		$this->events[] = array($column, $serialised_prikey, $event);
	}
}

foreach (array(PHPOF2_DB::LOB_STORAGE_DB, PHPOF2_DB::LOB_STORAGE_FS) as $storage_type) {
	$db_options = array(
		'lob_storage_fs_path' => dirname(__FILE__) . '/lobs/%t',
		'lob_storage' => $storage_type,
	);
	$db = PHPOF2_DB::factory($dsn, $db_options);

	$table = new PHPOF2_DBTable($db, 'lob_test');
	$all = $table->getAll();
	$row = $table->createRowObject();
	while ($all->fetchIntoObject($row)) {
		$row->delete();
	}

	foreach(array('test1', 'test2') as $name) {
		$row = $table->createRowObject();
		
		$storage = $row->getLOBStorage('data');
		if ($storage !== false) {
			throw new Exception("Error when testing getLOBStorage(): returned '$storage' instead of the expected FALSE for row ID $id");
		}
		
		$row->name = $name;
		$row->insert();
		
		$storage = $row->getLOBStorage('data');
		if ($storage !== null) {
			throw new Exception("Error when testing getLOBStorage(): returned '$storage' instead of the expected FALSE for row ID $id");
		}
		
		$data_raw = file_get_contents(dirname(__FILE__) . "/$name.png");
		$row->storeLOBData($data_raw, 'data');
		$id = $row->id;
	
		$row = $table->createRowObject();
		$row->get($id);
		if ($row->data != PHPOF2_DB::LOB_PLACEHOLDER) {
			throw new Exception("Error when testing LOBs with storage type '$storage_type': no placeholder after retrieval of row ID $id");
		}
		if ($row->getLOBData('data') != $data_raw) {
			throw new Exception("Error when testing LOBs with storage type '$storage_type': data does not match after retrieval of row ID $id");
		}
		$storage = $row->getLOBStorage('data');
		if ($storage != $storage_type) {
			throw new Exception("Error when testing getLOBStorage(): returned '$storage' instead of the expected '$storage_type' for row ID $id");
		}
	}
}
$res = $db->query("SELECT id FROM lob_test LIMIT 1");
$a_row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
$row->get($a_row['id']);

$table->migrateLOBStorage(PHPOF2_DB::LOB_STORAGE_DB);

$lobtest = new PHPOF2_LOB_Test();
$table->setMigrateLOBStorageCallback(array($lobtest, 'migrateLOBStorageCallback'));
$data = $row->getLOBData('data');
$row->storeLOBData($data, 'data', PHPOF2_DB::LOB_STORAGE_FS);

// this should now only migrate the second row; we've already done one
$table->migrateLOBStorage(PHPOF2_DB::LOB_STORAGE_FS);

$fs_path = $table->getLOBStorageFSPath('data');
$filename = $row->getLOBStorageFSFilename('data');
$filename = $fs_path . '/' . $filename;
$row->delete();
if (file_exists($filename)) {
	throw new Exception("Error when deleting row ID " . $a_row['id'] . "; the filesystem storage file '$filename' is still there");
}
if ($lobtest->events[0][2] != PHPOF2_DBTable::LOB_MIGRATION_EVENT_SKIP) {
	var_dump($table->events);
	throw new Exception("Error when migrating back to FS; we should have skipped the first row");
}
if ($lobtest->events[1][2] != PHPOF2_DBTable::LOB_MIGRATION_EVENT_START) {
	var_dump($table->events);
	throw new Exception("Error when migrating back to FS; we should have done the second row");
}
